@@ -8,16 +8,42 @@ from typing import Any
 from uuid import uuid4
 
 import anthropic
+import httpx
 from anthropic.types import TextBlock, MessageParam
 from dotenv import load_dotenv
 
 from silica.developer.context import AgentContext
+from silica.developer.models import ModelSpec
 from silica.developer.prompt import create_system_message
 from silica.developer.rate_limiter import RateLimiter
 from silica.developer.toolbox import Toolbox
 from silica.developer.sandbox import DoSomethingElseError
 
 
+def get_thinking_config(thinking_mode: str, model_spec: ModelSpec) -> dict | None:
+    """Get the thinking configuration for the API call based on the current mode.
+
+    Args:
+        thinking_mode: One of "off", "normal", or "ultra"
+        model_spec: The model specification dict
+
+    Returns:
+        Thinking config dict for API call, or None if thinking is off or unsupported
+    """
+    # Check if model supports thinking
+    if not model_spec.get("thinking_support", False):
+        return {"type": "disabled"}
+
+    if thinking_mode == "off":
+        return None
+    elif thinking_mode == "normal":
+        return {"type": "enabled", "budget_tokens": 8000}
+    elif thinking_mode == "ultra":
+        return {"type": "enabled", "budget_tokens": 20000}
+    else:
+        return {"type": "disabled"}
+
+
 def retry_with_exponential_backoff(func, max_retries=5, base_delay=1, max_delay=60):
     def wrapper(*args, **kwargs):
         retries = 0
@@ -202,94 +228,6 @@ def _inline_latest_file_mentions(
     return results
 
 
-def _apply_compaction_transition(
-    agent_context: AgentContext, transition
-) -> AgentContext:
-    """Apply a compaction transition to the agent context.
-
-    Args:
-        agent_context: The current agent context
-        transition: CompactionTransition object with new session info
-
-    Returns:
-        Updated agent context with compacted conversation
-    """
-    # Update session IDs to reflect the transition
-    agent_context.parent_session_id = transition.original_session_id
-    agent_context.session_id = transition.new_session_id
-
-    # Replace chat history with compacted version
-    agent_context._chat_history = transition.compacted_messages.copy()
-
-    # Clear tool result buffer for clean state
-    agent_context.tool_result_buffer.clear()
-
-    return agent_context
-
-
-def _check_and_apply_compaction(
-    agent_context: AgentContext,
-    model: dict,
-    user_interface,
-    enable_compaction: bool = True,
-) -> tuple[AgentContext, bool]:
-    """Check if compaction is needed and apply it if necessary.
-
-    Args:
-        agent_context: The agent context to check
-        model: Model specification dict
-        user_interface: User interface for notifications
-        enable_compaction: Whether compaction is enabled
-
-    Returns:
-        Tuple of (possibly updated agent_context, True if compaction was applied)
-    """
-    if not enable_compaction:
-        return agent_context, False
-
-    # Only check compaction when conversation state is complete
-    # (no pending tool results and conversation has actual content)
-    if (
-        agent_context.tool_result_buffer
-        or not agent_context.chat_history
-        or len(agent_context.chat_history) <= 2
-    ):
-        return agent_context, False
-
-    try:
-        from silica.developer.compacter import ConversationCompacter
-
-        compacter = ConversationCompacter()
-        model_name = model["title"]
-
-        transition = compacter.compact_and_transition(agent_context, model_name)
-
-        if transition:
-            # Apply the transition to start using the compacted conversation
-            updated_context = _apply_compaction_transition(agent_context, transition)
-
-            # Notify user about the compaction and transition
-            user_interface.handle_system_message(
-                f"[bold green]Conversation compacted: "
-                f"{transition.summary.original_message_count} messages → "
-                f"new session {transition.new_session_id[:8]}[/bold green]",
-                markdown=False,
-            )
-
-            # Save both sessions: original (already saved) and new compacted one
-            updated_context.flush(updated_context.chat_history, compact=False)
-            return updated_context, True
-
-    except Exception as e:
-        # Log compaction errors but continue normally
-        user_interface.handle_system_message(
-            f"[yellow]Compaction check failed: {e}[/yellow]",
-            markdown=False,
-        )
-
-    return agent_context, False
-
-
 def _continuation_message(final_message: MessageParam) -> MessageParam | None:
     continue_message = {
         "type": "text",
@@ -315,18 +253,26 @@ async def run(
     initial_prompt: str = None,
     single_response: bool = False,
     tool_names: list[str] | None = None,
+    tools: list | None = None,
     system_prompt: dict[str, Any] | None = None,
     enable_compaction: bool = True,
+    log_file_path: str | None = None,
 ) -> list[MessageParam]:
     load_dotenv()
     user_interface, model = (
         agent_context.user_interface,
         agent_context.model_spec,
     )
-    toolbox = Toolbox(agent_context, tool_names=tool_names)
+    # Create toolbox with either tools or tool_names (tools takes precedence)
+    toolbox = Toolbox(agent_context, tool_names=tool_names, tools=tools)
     if hasattr(user_interface, "set_toolbox"):
         user_interface.set_toolbox(toolbox)
 
+    # Initialize request/response logger if enabled
+    from silica.developer.request_logger import RequestResponseLogger
+
+    logger = RequestResponseLogger(log_file_path)
+
     api_key = os.getenv("ANTHROPIC_API_KEY")
     if not api_key:
         user_interface.handle_system_message(
@@ -353,11 +299,6 @@ async def run(
 
     while True:
         try:
-            # Check for compaction when conversation state is complete
-            agent_context, _ = _check_and_apply_compaction(
-                agent_context, model, user_interface, enable_compaction
-            )
-
             if return_to_user_after_interrupt or (
                 not agent_context.tool_result_buffer
                 and not single_response
@@ -365,10 +306,18 @@ async def run(
             ):
                 # Reset the interrupt flag
                 return_to_user_after_interrupt = False
-                cost = f"${agent_context.usage_summary()['total_cost']:.2f}"
+
                 user_input = ""
                 while not user_input.strip():
-                    user_input = await user_interface.get_user_input(f"{cost} > ")
+                    # Build prompt inside loop so it updates when thinking mode changes
+                    cost = f"${agent_context.usage_summary()['total_cost']:.2f}"
+                    prompt = f"{cost} > "
+                    if agent_context.thinking_mode == "normal":
+                        prompt = f"💭 {cost} > "
+                    elif agent_context.thinking_mode == "ultra":
+                        prompt = f"🧠 {cost} > "
+
+                    user_input = await user_interface.get_user_input(prompt)
 
                 command_name = (
                     user_input.split()[0][1:] if user_input.startswith("/") else ""
@@ -434,8 +383,18 @@ async def run(
                     agent_context.tool_result_buffer.clear()
                     agent_context.flush(
                         agent_context.chat_history,
-                        compact=False,  # Compaction handled explicitly above
+                        compact=False,  # Compaction handled explicitly below
                     )
+
+                # Check for compaction after tool results are converted to messages
+                # This ensures we have the complete conversation state including tool interactions
+                from silica.developer.compacter import ConversationCompacter
+
+                compacter = ConversationCompacter(client=client, logger=logger)
+                agent_context, _ = compacter.check_and_apply_compaction(
+                    agent_context, model["title"], user_interface, enable_compaction
+                )
+
                 initial_prompt = None
 
             system_message = create_system_message(
@@ -453,16 +412,6 @@ async def run(
                     try:
                         await rate_limiter.check_and_wait(user_interface)
 
-                        # Check for compaction before making API call - this is the critical timing
-                        # where we can catch conversations that have grown too large
-                        agent_context, compaction_applied = _check_and_apply_compaction(
-                            agent_context, model, user_interface, enable_compaction
-                        )
-                        if compaction_applied:
-                            # If compaction occurred, restart the API request attempt
-                            # with the newly compacted conversation
-                            continue
-
                         # Calculate conversation size before sending the next request
                         # This ensures we have a complete conversation state for accurate counting
                         conversation_size_for_display = None
@@ -473,7 +422,9 @@ async def run(
                                     ConversationCompacter,
                                 )
 
-                                compacter = ConversationCompacter()
+                                compacter = ConversationCompacter(
+                                    client=client, logger=logger
+                                )
                                 model_name = model["title"]
 
                                 # Check if conversation has incomplete tool_use before counting tokens
@@ -512,24 +463,128 @@ async def run(
                         messages = _inline_latest_file_mentions(
                             agent_context.chat_history
                         )
-                        with client.messages.stream(
-                            system=system_message,
-                            max_tokens=model["max_tokens"],
+
+                        # Get thinking configuration if enabled
+                        thinking_config = get_thinking_config(
+                            agent_context.thinking_mode, model
+                        )
+
+                        # Calculate max_tokens based on whether thinking is enabled
+                        # When thinking is enabled, max_tokens must be thinking_budget + completion_tokens
+                        max_tokens = model["max_tokens"]
+                        if thinking_config and thinking_config.get("type") == "enabled":
+                            max_tokens = (
+                                thinking_config["budget_tokens"] + model["max_tokens"]
+                            )
+
+                        api_kwargs = {
+                            "system": system_message,
+                            "max_tokens": max_tokens,
+                            "messages": messages,
+                            "model": model["title"],
+                            "tools": toolbox.agent_schema,
+                        }
+
+                        # Add thinking parameter if configured
+                        if thinking_config:
+                            api_kwargs["thinking"] = thinking_config
+
+                        # Log the request
+                        logger.log_request(
                             messages=messages,
+                            system_message=system_message,
                             model=model["title"],
+                            max_tokens=max_tokens,
                             tools=toolbox.agent_schema,
-                        ) as stream:
-                            for chunk in stream:
-                                if chunk.type == "text":
-                                    ai_response += chunk.text
+                            thinking_config=thinking_config,
+                        )
 
-                            final_message = stream.get_final_message()
+                        thinking_content = ""
+                        try:
+                            with client.messages.stream(**api_kwargs) as stream:
+                                for chunk in stream:
+                                    if chunk.type == "text":
+                                        ai_response += chunk.text
+                                    elif chunk.type == "content_block_start":
+                                        # Check if this is a thinking block
+                                        if hasattr(chunk, "content_block") and hasattr(
+                                            chunk.content_block, "type"
+                                        ):
+                                            if chunk.content_block.type == "thinking":
+                                                thinking_content = ""
+                                    elif chunk.type == "content_block_delta":
+                                        # Accumulate thinking content if this is a thinking delta
+                                        if hasattr(chunk, "delta") and hasattr(
+                                            chunk.delta, "type"
+                                        ):
+                                            if chunk.delta.type == "thinking_delta":
+                                                thinking_content += chunk.delta.thinking
 
-                        rate_limiter.update(stream.response.headers)
-                        break
+                                final_message = stream.get_final_message()
+
+                            # Log the response
+                            logger.log_response(
+                                message=final_message,
+                                usage=final_message.usage,
+                                stop_reason=final_message.stop_reason,
+                                thinking_content=thinking_content
+                                if thinking_content
+                                else None,
+                            )
+
+                            rate_limiter.update(stream.response.headers)
+                            break
+                        except (
+                            httpx.RemoteProtocolError,
+                            httpx.ReadError,
+                            httpx.ConnectError,
+                            httpx.NetworkError,
+                        ) as e:
+                            # Handle network/connection errors during streaming
+                            logger.log_error(
+                                error_type=type(e).__name__,
+                                error_message=str(e),
+                                context={
+                                    "attempt": attempt + 1,
+                                    "max_retries": max_retries,
+                                },
+                            )
+
+                            if attempt == max_retries - 1:
+                                user_interface.handle_system_message(
+                                    f"[bold red]Network error during streaming: {str(e)}. Max retries reached.[/bold red]",
+                                    markdown=False,
+                                )
+                                raise
+
+                            delay = min(
+                                base_delay * (2**attempt) + random.uniform(0, 1),
+                                max_delay,
+                            )
+                            user_interface.handle_system_message(
+                                f"[bold yellow]Network error during streaming. Retrying in {delay:.2f} seconds... (Attempt {attempt + 1}/{max_retries})[/bold yellow]",
+                                markdown=False,
+                            )
+                            time.sleep(delay)
+                            # Clear partial response before retrying
+                            ai_response = ""
+                            thinking_content = ""
+                            continue
                     except anthropic.RateLimitError as e:
                         # Handle rate limit errors specifically
                         backoff_time = rate_limiter.handle_rate_limit_error(e)
+
+                        # Log the error
+                        logger.log_error(
+                            error_type="RateLimitError",
+                            error_message=str(e),
+                            context={
+                                "attempt": attempt + 1,
+                                "max_retries": max_retries,
+                                "backoff_time": backoff_time,
+                            },
+                        )
+
                         if attempt == max_retries - 1:
                             user_interface.handle_system_message(
                                 "[bold red]Rate limit exceeded. Max retries reached. Please try again later.[/bold red]",
@@ -538,7 +593,7 @@ async def run(
                             raise
 
                         user_interface.handle_system_message(
-                            f"[bold yellow]Rate limit exceeded. Retrying in {backoff_time:.2f} seconds... (Attempt {attempt+1}/{max_retries})[/bold yellow]",
+                            f"[bold yellow]Rate limit exceeded. Retrying in {backoff_time:.2f} seconds... (Attempt {attempt + 1}/{max_retries})[/bold yellow]",
                             markdown=False,
                         )
                         # Wait time is already set in handle_rate_limit_error
@@ -546,6 +601,18 @@ async def run(
 
                     except anthropic.APIStatusError as e:
                         rate_limiter.update(e.response.headers)
+
+                        # Log the error
+                        logger.log_error(
+                            error_type="APIStatusError",
+                            error_message=str(e),
+                            context={
+                                "attempt": attempt + 1,
+                                "max_retries": max_retries,
+                                "status_code": getattr(e, "status_code", None),
+                            },
+                        )
+
                         if attempt == max_retries - 1:
                             raise
                         if "Overloaded" in str(e):
@@ -579,6 +646,16 @@ async def run(
 
             agent_context.report_usage(final_message.usage)
             usage_summary = agent_context.usage_summary()
+
+            # Display thinking content if present
+            if thinking_content:
+                thinking_tokens = usage_summary.get("total_thinking_tokens", 0)
+                thinking_cost = usage_summary.get("thinking_cost", 0.0)
+                if hasattr(user_interface, "handle_thinking_content"):
+                    user_interface.handle_thinking_content(
+                        thinking_content, thinking_tokens, thinking_cost, collapsed=True
+                    )
+
             user_interface.handle_assistant_message(ai_response)
 
             # Use conversation size calculated before the API call (when state was complete)
@@ -595,6 +672,8 @@ async def run(
                 cached_tokens=usage_summary["cached_tokens"],
                 conversation_size=conversation_size,
                 context_window=context_window,
+                thinking_tokens=usage_summary.get("total_thinking_tokens", 0),
+                thinking_cost=usage_summary.get("thinking_cost", 0.0),
             )
 
             if final_message.stop_reason == "tool_use":
@@ -613,6 +692,14 @@ async def run(
                     # Add all results to buffer and display them
                     for tool_use, result in zip(tool_uses, results):
                         tool_name = getattr(tool_use, "name", "unknown_tool")
+
+                        # Log tool execution
+                        logger.log_tool_execution(
+                            tool_name=tool_name,
+                            tool_input=getattr(tool_use, "input", {}),
+                            tool_result=result,
+                        )
+
                         agent_context.tool_result_buffer.append(result)
                         user_interface.handle_tool_result(tool_name, result)
                 except KeyboardInterrupt:
@@ -765,14 +852,15 @@ async def run(
 
             if interrupt_count >= 2:
                 user_interface.handle_system_message(
-                    "[bold red]Double interrupt detected. Exiting...[/bold red]",
+                    "[bold red]Exiting...[/bold red]",
                     markdown=False,
                 )
+                # Clean exit - break from loop to allow proper cleanup
                 break
             else:
                 user_interface.handle_system_message(
                     "[bold yellow]"
-                    "KeyboardInterrupt detected. Press Ctrl+C again to exit, or continue typing to resume."
+                    "Interrupted. Press Ctrl+C again within 1 second to exit, or press Enter to continue."
                     "[/bold yellow]",
                     markdown=False,
                 )
