@@ -36,6 +36,10 @@ class AgentContext:
     usage: list[tuple[Any, Any]]
     memory_manager: "MemoryManager"
     cli_args: list[str] = None
+    thinking_mode: str = "off"  # "off", "normal", or "ultra"
+    history_base_dir: Path | None = (
+        None  # Base directory for history (defaults to ~/.silica/personas/default)
+    )
     _chat_history: list[MessageParam] = None
     _tool_result_buffer: list[dict] = None
 
@@ -46,6 +50,31 @@ class AgentContext:
         if self._tool_result_buffer is None:
             self._tool_result_buffer = []
 
+    def _get_history_dir(self) -> Path:
+        """Get the history directory for this context.
+
+        Uses history_base_dir if provided, otherwise defaults to ~/.silica/personas/default.
+        For root contexts, returns base/history/{session_id}.
+        For sub-agent contexts, returns base/history/{parent_session_id}.
+
+        Returns:
+            Path to the history directory for this context
+        """
+        # Ensure history_base_dir is a Path object
+        if self.history_base_dir:
+            base = (
+                Path(self.history_base_dir)
+                if not isinstance(self.history_base_dir, Path)
+                else self.history_base_dir
+            )
+        else:
+            base = Path.home() / ".silica" / "personas" / "default"
+
+        context_dir = (
+            self.parent_session_id if self.parent_session_id else self.session_id
+        )
+        return base / "history" / context_dir
+
     @property
     def chat_history(self) -> list[MessageParam]:
         """Get the chat history."""
@@ -64,6 +93,7 @@ class AgentContext:
         user_interface: UserInterface,
         session_id: str = None,
         cli_args: list[str] = None,
+        persona_base_directory: Path = None,
     ) -> "AgentContext":
         sandbox = Sandbox(
             sandbox_contents[0] if sandbox_contents else os.getcwd(),
@@ -72,7 +102,7 @@ class AgentContext:
             permission_check_rendering_callback=user_interface.permission_rendering_callback,
         )
 
-        memory_manager = MemoryManager()
+        memory_manager = MemoryManager(base_dir=persona_base_directory / "memory")
 
         # Use provided session_id or generate a new one
         context_session_id = session_id if session_id else str(uuid4())
@@ -86,12 +116,15 @@ class AgentContext:
             usage=[],
             memory_manager=memory_manager,
             cli_args=cli_args.copy() if cli_args else None,
+            history_base_dir=persona_base_directory,
         )
 
         # If a session_id was provided, attempt to load that session
         if session_id:
-            # Load the session data
-            loaded_context = load_session_data(session_id, context)
+            # Load the session data - pass persona_base_directory to load from correct location
+            loaded_context = load_session_data(
+                session_id, context, history_base_dir=persona_base_directory
+            )
 
             # If loading was successful, update message count for UI feedback
             if loaded_context and loaded_context.chat_history:
@@ -107,10 +140,10 @@ class AgentContext:
         return context
 
     def with_user_interface(
-        self, user_interface: UserInterface, keep_history=False
+        self, user_interface: UserInterface, keep_history=False, session_id: str = None
     ) -> "AgentContext":
         return AgentContext(
-            session_id=str(uuid4()),
+            session_id=session_id if session_id else str(uuid4()),
             parent_session_id=self.session_id,
             model_spec=self.model_spec,
             sandbox=self.sandbox,
@@ -118,6 +151,7 @@ class AgentContext:
             usage=self.usage,
             memory_manager=self.memory_manager,
             cli_args=self.cli_args.copy() if self.cli_args else None,
+            history_base_dir=self.history_base_dir,  # Preserve history_base_dir
             _chat_history=self.chat_history.copy() if keep_history else [],
             _tool_result_buffer=self.tool_result_buffer.copy() if keep_history else [],
         )
@@ -170,7 +204,9 @@ class AgentContext:
         usage_summary = {
             "total_input_tokens": 0,
             "total_output_tokens": 0,
+            "total_thinking_tokens": 0,
             "total_cost": 0.0,
+            "thinking_cost": 0.0,
             "cached_tokens": 0,
             "model_breakdown": {},
         }
@@ -179,6 +215,7 @@ class AgentContext:
             model_name = model_spec["title"]
             pricing = model_spec["pricing"]
             cache_pricing = model_spec["cache_pricing"]
+            thinking_pricing = model_spec.get("thinking_pricing", {"thinking": 0.0})
 
             input_tokens = self._prop_or_dict_entry(usage_entry, "input_tokens")
             output_tokens = self._prop_or_dict_entry(usage_entry, "output_tokens")
@@ -189,22 +226,35 @@ class AgentContext:
                 usage_entry, "cache_read_input_tokens"
             )
 
+            # Extract thinking tokens if present (Anthropic SDK v0.62.0+)
+            thinking_tokens = 0
+            if hasattr(usage_entry, "thinking_tokens"):
+                thinking_tokens = usage_entry.thinking_tokens
+            elif isinstance(usage_entry, dict) and "thinking_tokens" in usage_entry:
+                thinking_tokens = usage_entry["thinking_tokens"]
+
             usage_summary["total_input_tokens"] += input_tokens
             usage_summary["total_output_tokens"] += output_tokens
+            usage_summary["total_thinking_tokens"] += thinking_tokens
             usage_summary["cached_tokens"] += cache_read_input_tokens
 
+            thinking_cost = thinking_tokens * thinking_pricing["thinking"]
+
             total_cost = (
                 input_tokens * pricing["input"]
                 + output_tokens * pricing["output"]
                 + cache_pricing["read"] * cache_read_input_tokens
                 + cache_pricing["write"] * cache_creation_input_tokens
+                + thinking_cost
             )
 
             if model_name not in usage_summary["model_breakdown"]:
                 usage_summary["model_breakdown"][model_name] = {
                     "total_input_tokens": 0,
                     "total_output_tokens": 0,
+                    "total_thinking_tokens": 0,
                     "total_cost": 0.0,
+                    "thinking_cost": 0.0,
                     "cached_tokens": 0,
                     "token_breakdown": {},
                 }
@@ -212,24 +262,86 @@ class AgentContext:
             model_breakdown = usage_summary["model_breakdown"][model_name]
             model_breakdown["total_input_tokens"] += input_tokens
             model_breakdown["total_output_tokens"] += output_tokens
+            model_breakdown["total_thinking_tokens"] += thinking_tokens
             model_breakdown["cached_tokens"] += cache_read_input_tokens
 
             model_breakdown["total_cost"] += total_cost
+            model_breakdown["thinking_cost"] += thinking_cost / 1_000_000
 
             usage_summary["total_cost"] += total_cost
+            usage_summary["thinking_cost"] += thinking_cost
 
         usage_summary["total_cost"] /= 1_000_000
+        usage_summary["thinking_cost"] /= 1_000_000
 
         return usage_summary
 
+    def rotate(
+        self,
+        archive_suffix: str,
+        new_messages: list[MessageParam],
+        compaction_metadata: dict | None = None,
+    ) -> str:
+        """Archive the current conversation and update this context with new messages.
+
+        This method mutates the context in place:
+        1. Archives the current root.json to a timestamped archive file
+        2. Updates this context's chat history with the provided messages
+        3. Clears the tool result buffer
+        4. Optionally stores compaction metadata for flush()
+
+        Args:
+            archive_suffix: Suffix for the archive filename (e.g., "pre-compaction-20250112_140530")
+            new_messages: The new messages to use in the rotated context
+            compaction_metadata: Optional metadata about compaction (stored for next flush())
+
+        Returns:
+            str: The archive filename
+
+        Raises:
+            ValueError: If called on a sub-agent context (which doesn't have root.json)
+        """
+        if self.parent_session_id is not None:
+            raise ValueError(
+                "rotate() can only be called on root contexts, not sub-agent contexts"
+            )
+
+        # Use the parameterized history directory
+        history_dir = self._get_history_dir()
+        root_file = history_dir / "root.json"
+        archive_file = history_dir / f"{archive_suffix}.json"
+
+        # Archive existing root.json if it exists
+        if root_file.exists():
+            try:
+                with open(root_file, "r") as f:
+                    existing_data = json.load(f)
+                with open(archive_file, "w") as f:
+                    json.dump(existing_data, f, indent=2)
+            except (json.JSONDecodeError, FileNotFoundError) as e:
+                raise RuntimeError(f"Failed to archive conversation: {e}") from e
+
+        # Update this context in place
+        self._chat_history = new_messages
+        self._tool_result_buffer.clear()
+        self.flush(new_messages, compact=False)
+
+        # Store compaction metadata if provided
+        if compaction_metadata:
+            self._compaction_metadata = compaction_metadata
+
+        return f"{archive_suffix}.json"
+
     def flush(self, chat_history, compact=True):
         """Save the agent context and chat history to a file.
 
         For root contexts (parent_session_id is None), saves to:
-            ~/.hdev/history/{session_id}/root.json
+            {history_base_dir}/history/{session_id}/root.json
 
         For sub-agent contexts (parent_session_id is not None), saves to:
-            ~/.hdev/history/{parent_session_id}/{session_id}.json
+            {history_base_dir}/history/{parent_session_id}/{session_id}.json
+
+        Where history_base_dir defaults to ~/.silica/personas/default if not specified.
 
         Args:
             chat_history: The chat history to save
@@ -243,17 +355,9 @@ class AgentContext:
 
         # Note: Compaction is now handled explicitly in the agent loop rather than
         # as a side effect of flush. This allows for proper session transitions.
-        compaction_summary = None
 
-        # Base history directory
-        history_dir = Path.home() / ".hdev" / "history"
-
-        # For root contexts, use their own session_id
-        # For sub-agent contexts, use the parent_session_id
-        context_dir = (
-            self.parent_session_id if self.parent_session_id else self.session_id
-        )
-        history_dir = history_dir / context_dir
+        # Use the parameterized history directory
+        history_dir = self._get_history_dir()
 
         # Create the directory if it doesn't exist
         history_dir.mkdir(parents=True, exist_ok=True)
@@ -264,6 +368,9 @@ class AgentContext:
         )
         history_file = history_dir / filename
 
+        # Get compaction metadata if present
+        compaction_metadata = getattr(self, "_compaction_metadata", None)
+
         # Get the current time for metadata
         current_time = datetime.now(timezone.utc).isoformat()
 
@@ -297,6 +404,7 @@ class AgentContext:
             "model_spec": self.model_spec,
             "usage": self.usage,
             "messages": chat_history,
+            "thinking_mode": self.thinking_mode,
             "metadata": {
                 "created_at": current_time,
                 "last_updated": current_time,
@@ -306,19 +414,19 @@ class AgentContext:
         }
 
         # Add compaction metadata if available
-        if compaction_summary:
+        if compaction_metadata:
             context_data["compaction"] = {
-                "original_session_id": self.parent_session_id,
-                "original_message_count": compaction_summary.original_message_count,
-                "original_token_count": compaction_summary.original_token_count,
-                "summary_token_count": compaction_summary.summary_token_count,
-                "compaction_ratio": compaction_summary.compaction_ratio,
-                "timestamp": str(
-                    Path(history_file).stat().st_mtime
-                    if Path(history_file).exists()
-                    else None
-                ),
+                "is_compacted": True,
+                "original_message_count": compaction_metadata.original_message_count,
+                "original_token_count": compaction_metadata.original_token_count,
+                "compacted_message_count": compaction_metadata.compacted_message_count,
+                "summary_token_count": compaction_metadata.summary_token_count,
+                "compaction_ratio": compaction_metadata.compaction_ratio,
+                "timestamp": current_time,
+                "pre_compaction_archive": compaction_metadata.archive_name,
             }
+            # Clear the metadata after using it
+            del self._compaction_metadata
 
         # If the file already exists, read it to preserve the original created_at time
         if os.path.exists(history_file):
@@ -347,7 +455,9 @@ class AgentContext:
 
 
 def load_session_data(
-    session_id: str, base_context: Optional[AgentContext] = None
+    session_id: str,
+    base_context: Optional[AgentContext] = None,
+    history_base_dir: Optional[Path] = None,
 ) -> Optional[AgentContext]:
     """
     Load session data from a file and return an updated AgentContext.
@@ -359,11 +469,17 @@ def load_session_data(
         session_id: The ID of the session to load
         base_context: Optional existing AgentContext to update with session data.
                       If not provided, a new context will be created.
+        history_base_dir: Optional base directory for history (defaults to ~/.silica/personas/default)
 
     Returns:
         Updated AgentContext if successful, None if loading failed
     """
-    history_dir = Path.home() / ".hdev" / "history" / session_id
+    base = (
+        history_base_dir
+        if history_base_dir
+        else (Path.home() / ".silica" / "personas" / "default")
+    )
+    history_dir = base / "history" / session_id
     root_file = history_dir / "root.json"
 
     if not root_file.exists():
@@ -390,6 +506,7 @@ def load_session_data(
         model_spec = session_data.get("model_spec", base_context.model_spec)
         parent_id = session_data.get("parent_session_id")
         cli_args = session_data.get("metadata", {}).get("cli_args")
+        thinking_mode = session_data.get("thinking_mode", "off")
 
         # Create a new context with the loaded data
         updated_context = AgentContext(
@@ -401,6 +518,8 @@ def load_session_data(
             usage=usage_data if usage_data else base_context.usage,
             memory_manager=base_context.memory_manager,
             cli_args=cli_args.copy() if cli_args else None,
+            thinking_mode=thinking_mode,
+            history_base_dir=history_base_dir,  # Preserve the history_base_dir
             _chat_history=chat_history,
             _tool_result_buffer=[],  # Always start with empty tool buffer
         )
