@@ -3,6 +3,7 @@ import asyncio
 import io
 import os
 import re
+from pathlib import Path
 from typing import Dict, Any, Annotated, Optional
 
 import cyclopts
@@ -16,6 +17,7 @@ from prompt_toolkit import PromptSession, ANSI
 from prompt_toolkit.history import FileHistory
 from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
 from prompt_toolkit.document import Document
+from prompt_toolkit.key_binding import KeyBindings
 
 from silica.developer import personas
 from silica.developer.agent_loop import run
@@ -31,7 +33,6 @@ from silica.developer.user_interface import UserInterface
 from silica.developer.toolbox import Toolbox
 from prompt_toolkit.completion import Completer, WordCompleter, Completion
 
-from silica.developer.utils import wrap_text_as_content_block
 
 SANDBOX_MODE_MAP = {mode.name.lower(): mode for mode in SandboxMode}
 SANDBOX_MODE_MAP["dwr"] = SandboxMode.ALLOW_ALL
@@ -44,6 +45,103 @@ def parse_sandbox_mode(value: str) -> SandboxMode:
     raise argparse.ArgumentTypeError(f"Invalid sandbox mode: {value}")
 
 
+def _find_git_root() -> Optional[Path]:
+    """Find the git repository root, if in a git repository."""
+    try:
+        current_dir = Path.cwd().resolve()
+
+        # Walk up directories looking for .git folder
+        while current_dir != current_dir.parent:
+            if (current_dir / ".git").is_dir():
+                return current_dir
+            current_dir = current_dir.parent
+
+        return None
+    except Exception:
+        return None
+
+
+def _get_persona_file_location() -> Path:
+    """Get the location where .persona file should be stored.
+
+    If in a git repository, use the git root. Otherwise, use the current directory.
+    """
+    git_root = _find_git_root()
+    if git_root:
+        return git_root / ".persona"
+    else:
+        return Path.cwd() / ".persona"
+
+
+def _read_persona_file() -> Optional[str]:
+    """Read the persona name from .persona file if it exists.
+
+    Returns:
+        Persona name from file, or None if file doesn't exist or is invalid
+    """
+    persona_file = _get_persona_file_location()
+
+    if not persona_file.exists():
+        return None
+
+    try:
+        persona_name = persona_file.read_text().strip()
+        if persona_name:
+            return persona_name
+        return None
+    except Exception:
+        return None
+
+
+def _write_persona_file(persona_name: str) -> None:
+    """Write the persona name to .persona file.
+
+    Args:
+        persona_name: Name of the persona to save
+    """
+    persona_file = _get_persona_file_location()
+
+    try:
+        persona_file.write_text(persona_name + "\n")
+    except Exception as e:
+        # Don't fail if we can't write the file, just log it
+        console = Console()
+        console.print(f"[yellow]Warning: Could not write .persona file: {e}[/yellow]")
+
+
+def _ensure_persona_in_gitignore() -> None:
+    """Ensure .persona is in .gitignore if in a git repository."""
+    git_root = _find_git_root()
+
+    if not git_root:
+        return  # Not in a git repository
+
+    gitignore_file = git_root / ".gitignore"
+
+    try:
+        # Read existing .gitignore
+        if gitignore_file.exists():
+            gitignore_content = gitignore_file.read_text()
+        else:
+            gitignore_content = ""
+
+        # Check if .persona is already in .gitignore
+        lines = gitignore_content.splitlines()
+        for line in lines:
+            if line.strip() == ".persona":
+                return  # Already present
+
+        # Add .persona to .gitignore
+        if gitignore_content and not gitignore_content.endswith("\n"):
+            gitignore_content += "\n"
+        gitignore_content += ".persona\n"
+
+        gitignore_file.write_text(gitignore_content)
+    except Exception:
+        # Don't fail if we can't update .gitignore, just skip it
+        pass
+
+
 # Use the pre-defined HORIZONTALS box which has only top and bottom borders
 # This makes it easier to copy-paste content from the terminal
 HORIZONTAL_ONLY_BOX = HORIZONTALS
@@ -79,19 +177,63 @@ def rich_to_prompt_toolkit(rich_text):
 
 
 class CLIUserInterface(UserInterface):
-    def __init__(self, console: Console, sandbox_mode: SandboxMode):
+    def __init__(self, console: Console, sandbox_mode: SandboxMode, agent_context=None):
         self.console = console
         self.sandbox_mode = sandbox_mode
         self.toolbox = None  # Will be set after Sandbox is created
+        self.agent_context = (
+            agent_context  # Reference to agent context for thinking mode
+        )
 
         # Initialize the session with the history file
         history_file_path = self._get_history_file_path()
         history = FileHistory(history_file_path)
+
+        # Set up key bindings
+        kb = KeyBindings()
+
+        # Store a reference to track mode switches and preserve user input
+        self._mode_switch_pending = False
+        self._preserved_input = None
+
+        @kb.add("c-t")
+        def _(event):
+            """Cycle through thinking modes: off -> normal -> ultra -> off
+
+            This handler cycles the thinking mode and preserves the current input.
+            The prompt will be re-rendered with the new mode icon.
+
+            Note: Even if agent_context is None (during early initialization), we must
+            handle this event to prevent the default Ctrl+T behavior (transpose characters).
+            """
+            if not self.agent_context:
+                # Agent context not yet initialized - do nothing but prevent default behavior
+                # by explicitly handling the event (don't let it fall through to defaults)
+                pass
+            else:
+                current_mode = self.agent_context.thinking_mode
+                if current_mode == "off":
+                    self.agent_context.thinking_mode = "normal"
+                elif current_mode == "normal":
+                    self.agent_context.thinking_mode = "ultra"
+                else:  # ultra
+                    self.agent_context.thinking_mode = "off"
+
+                # Mark that we have a pending mode switch
+                self._mode_switch_pending = True
+
+                # Preserve the current input text
+                self._preserved_input = event.app.current_buffer.text
+
+                # Abort the current prompt to trigger re-prompt with new mode
+                event.app.exit(result="")
+
         self.session = PromptSession(
             history=history,
             auto_suggest=AutoSuggestFromHistory(),
             enable_history_search=True,
             complete_while_typing=True,
+            key_bindings=kb,
         )
 
     def _get_history_file_path(self) -> str:
@@ -279,13 +421,57 @@ class CLIUserInterface(UserInterface):
         # Create the header section with command name only (parameters section removed)
         header = Group(Text("Command:", style="bold blue"), Text(f"  {name}"))
 
-        # Create the result section - treat content as markdown for code blocks, etc.
+        # Create the result section - handle content blocks or plain text
         result_header = Text("Result:", style="bold green")
-        result_content = (
-            Markdown(content)
-            if isinstance(content, str) and markdown
-            else Text(str(content))
-        )
+
+        # Check if content is a list of content blocks (new format with images)
+        if isinstance(content, list):
+            result_parts = []
+            for block in content:
+                if isinstance(block, dict) and block.get("type") == "text":
+                    # Text block - render as markdown
+                    text_content = block.get("text", "")
+                    if markdown and text_content:
+                        result_parts.append(Markdown(text_content))
+                    else:
+                        result_parts.append(Text(text_content))
+                elif isinstance(block, dict) and block.get("type") == "image":
+                    # Image block - show summary instead of raw base64
+                    source = block.get("source", {})
+                    if source.get("type") == "base64":
+                        data = source.get("data", "")
+                        media_type = source.get("media_type", "image/unknown")
+
+                        # Calculate approximate size in KB
+                        # Base64 encoding increases size by ~33%, so decode size is roughly size * 0.75
+                        base64_size_kb = len(data) * 0.75 / 1024
+
+                        # Create a nice summary
+                        image_summary = Text.assemble(
+                            ("📷 ", "cyan"),
+                            ("Image ", "bold cyan"),
+                            (f"({media_type}, ~{base64_size_kb:.1f} KB)", "dim"),
+                        )
+                        result_parts.append(image_summary)
+                    else:
+                        result_parts.append(
+                            Text(
+                                f"[Image: {source.get('type', 'unknown')}]",
+                                style="cyan",
+                            )
+                        )
+                else:
+                    # Unknown block type - show as text
+                    result_parts.append(Text(str(block)))
+
+            result_content = Group(*result_parts) if result_parts else Text("")
+        else:
+            # Legacy string content
+            result_content = (
+                Markdown(content)
+                if isinstance(content, str) and markdown
+                else Text(str(content))
+            )
 
         # Group all components together
         display_group = Group(header, Text(""), result_header, result_content)
@@ -303,7 +489,22 @@ class CLIUserInterface(UserInterface):
 
     async def get_user_input(self, prompt: str = "") -> str:
         _console = Console(file=None)
-        user_input = await self.session.prompt_async(rich_to_prompt_toolkit(prompt))
+
+        # If we have preserved input from a mode switch, restore it
+        default_text = ""
+        if self._mode_switch_pending and self._preserved_input is not None:
+            default_text = self._preserved_input
+            self._preserved_input = None
+            self._mode_switch_pending = False
+
+        user_input = await self.session.prompt_async(
+            rich_to_prompt_toolkit(prompt), default=default_text
+        )
+
+        # Handle empty input (should not happen with mode switch anymore)
+        if not user_input.strip() and default_text:
+            # This means user cleared the preserved text and hit enter
+            return ""
 
         # Handle multi-line input
         if user_input.strip() == "{":
@@ -335,6 +536,8 @@ class CLIUserInterface(UserInterface):
         cached_tokens: int | None = None,
         conversation_size: int | None = None,
         context_window: int | None = None,
+        thinking_tokens: int | None = None,
+        thinking_cost: float | None = None,
     ) -> None:
         token_components = [
             ("Token Count:\n", "bold"),
@@ -343,9 +546,16 @@ class CLIUserInterface(UserInterface):
                 "cyan",
             ),
             (f"Completion: {completion_tokens}\n", "green"),
-            (f"Total: {total_tokens}\n", "yellow"),
         ]
 
+        # Add thinking tokens if present
+        if thinking_tokens and thinking_tokens > 0:
+            token_components.append(
+                (f"Thinking: {thinking_tokens} (${thinking_cost:.4f})\n", "magenta")
+            )
+
+        token_components.append((f"Total: {total_tokens}\n", "yellow"))
+
         # Add conversation size and context window information if available
         if conversation_size is not None and context_window is not None:
             # Calculate percentage of context window used
@@ -400,6 +610,45 @@ class CLIUserInterface(UserInterface):
         else:
             self.console.print(message)
 
+    def handle_thinking_content(
+        self, content: str, tokens: int, cost: float, collapsed: bool = True
+    ) -> None:
+        """Display thinking content in a collapsible panel."""
+        from rich.markdown import Markdown
+        from rich.text import Text
+        from rich.console import Group
+
+        # Prepare the header with token count and cost
+        header = Text.assemble(
+            ("Thinking Process ", "bold magenta"),
+            (f"({tokens} tokens, ${cost:.4f})", "dim"),
+        )
+
+        # If collapsed, show only first 150 characters
+        if collapsed and len(content) > 150:
+            preview = content[:150] + "..."
+            display_content = Text(preview, style="dim")
+            footer = Text(
+                "\n[Use /dump to see full thinking content]", style="dim italic"
+            )
+            content_group = Group(display_content, footer)
+        else:
+            # Show full content with markdown rendering
+            content_group = Markdown(content)
+
+        panel = create_clean_panel(
+            content_group,
+            title=str(header),
+            style="magenta",
+        )
+
+        self.console.print(panel)
+
+    def update_thinking_status(self, tokens: int, budget: int, cost: float) -> None:
+        """Update the status display with thinking progress (not implemented in CLI yet)."""
+        # This would require live status updates during streaming
+        # For now, we'll just track the final thinking content
+
 
 class CustomCompleter(Completer):
     def __init__(self, commands, history):
@@ -504,13 +753,49 @@ class CustomCompleter(Completer):
                     yield Completion(history_item, start_position=start_position)
 
 
-def sessions(workdir: Annotated[str, cyclopts.Parameter("workdir")]):
-    s = list_sessions(workdir=workdir)
+def sessions(
+    workdir: Annotated[Optional[str], cyclopts.Parameter("workdir")] = None,
+    *,
+    persona: Annotated[
+        Optional[str], cyclopts.Parameter(help="Persona to list sessions for")
+    ] = None,
+):
+    """List available developer sessions with metadata.
+
+    Args:
+        workdir: Optional working directory to filter sessions by
+        persona: Optional persona name to list sessions for (defaults to 'default')
+    """
+    # Get persona base directory if specified
+    history_base_dir = None
+    if persona:
+        persona_obj = personas.get_or_create(persona, interactive=False)
+        history_base_dir = persona_obj.base_directory
+
+    s = list_sessions(workdir=workdir, history_base_dir=history_base_dir)
     print_session_list(s)
 
 
-def resume(session_id: Annotated[str, cyclopts.Parameter("session_id")]):
-    resume_session(session_id)
+def resume(
+    session_id: Annotated[str, cyclopts.Parameter("session_id")],
+    *,
+    persona: Annotated[
+        Optional[str], cyclopts.Parameter(help="Persona that owns the session")
+    ] = None,
+):
+    """Resume a previous developer session.
+
+    Args:
+        session_id: Session ID or prefix to resume
+        persona: Optional persona name that owns the session (defaults to 'default')
+    """
+    # Get persona base directory if specified
+    history_base_dir = None
+    if persona:
+        persona_obj = personas.get_or_create(persona, interactive=False)
+        history_base_dir = persona_obj.base_directory
+
+    resume_session(session_id, history_base_dir=history_base_dir)
 
 
 def attach_tools(app):
@@ -521,6 +806,7 @@ def attach_tools(app):
         sandbox_mode=SandboxMode.ALLOW_ALL,
         sandbox_contents=[],
         user_interface=CLIUserInterface(console, sandbox.mode),
+        persona_base_directory=Path("~/.silica/personas/default").expanduser(),
     )
     toolbox = Toolbox(context)
 
@@ -572,6 +858,12 @@ def cyclopts_main(
     persona: Annotated[
         Optional[str], cyclopts.Parameter(help="Persona to use for the assistant")
     ] = None,
+    log_requests: Annotated[
+        Optional[str],
+        cyclopts.Parameter(
+            help="Path to log file for JSON request/response logging. If not specified, logging is disabled."
+        ),
+    ] = None,
 ):
     """
     Cyclopts-based reimplementation of the main method for the AI-powered developer assistant.
@@ -593,7 +885,7 @@ def cyclopts_main(
     """
 
     # Cache available options for validation
-    available_models = model_names()
+    model_names()
     available_personas = personas.names()
 
     # Store original args for session metadata (simulate sys.argv for compatibility)
@@ -616,16 +908,14 @@ def cyclopts_main(
         original_args.extend(["--session-id", session_id])
     if persona:
         original_args.extend(["--persona", persona])
+    if log_requests:
+        original_args.extend(["--log-requests", log_requests])
     if sandbox:
         original_args.extend(sandbox)
 
-    # Validate model choice
-    if model not in available_models:
-        console = Console()
-        console.print(
-            f"[red]Error: Invalid model '{model}'. Available models: {', '.join(available_models)}[/red]"
-        )
-        return
+    # Note: We no longer validate model choice here.
+    # If the model is not in available_models, get_model() will use Opus ModelSpec
+    # as a fallback, allowing custom model names while maintaining pricing/limits.
 
     # Parse sandbox mode with error handling
     try:
@@ -639,20 +929,62 @@ def cyclopts_main(
     if dwr:
         parsed_sandbox_mode = SandboxMode.ALLOW_ALL
 
-    # Validate persona if specified
-    if persona and persona not in available_personas:
-        console = Console()
+    # Initialize console early for validation messages
+    console = Console()
+
+    # Handle .persona file logic
+    # The .persona file allows setting a project-specific default persona
+    # while the CLI --persona argument provides temporary overrides without
+    # modifying the file. This enables both convenience and flexibility.
+    #
+    # Behavior:
+    # - CLI argument (--persona): Temporarily uses specified persona, file unchanged
+    # - No CLI argument + .persona file exists: Uses persona from file
+    # - No CLI argument + no file: Uses "default" persona, creates .persona file
+    #
+    # 1. Read existing .persona file if it exists
+    file_persona = _read_persona_file()
+
+    # 2. Determine which persona to use
+    # Priority: CLI argument > .persona file > "default"
+    if persona:
+        # CLI argument takes precedence (temporary override)
+        persona_name = persona
+    elif file_persona:
+        # Use persona from file if no CLI argument
+        persona_name = file_persona
+    else:
+        # Default to "default" persona
+        persona_name = "default"
+
+    # Validate persona
+    if persona_name not in available_personas:
         console.print(
-            f"[red]Error: Invalid persona '{persona}'. Available personas: {', '.join(available_personas)}[/red]"
+            f"[red]Error: Invalid persona '{persona_name}'. Available personas: {', '.join(available_personas)}[/red]"
         )
         return
 
-    # Check for session ID in environment variable if not specified in args
-    if not session_id and "silica_DEVELOPER_SESSION_ID" in os.environ:
-        session_id = os.environ.get("silica_DEVELOPER_SESSION_ID")
+    # Get or create persona (prompts user if needed)
+    try:
+        persona_obj = personas.get_or_create(persona_name, interactive=True)
+    except KeyboardInterrupt:
+        # User cancelled persona creation
+        console.print("\n[yellow]Persona creation cancelled. Exiting.[/yellow]")
+        return
 
-    # Initialize console and user interface (identical to original)
-    console = Console()
+    # 3. Write the chosen persona to .persona file ONLY if no CLI argument was provided
+    # This allows CLI arguments to temporarily override without changing the file
+    if not persona:
+        _write_persona_file(persona_name)
+
+    # 4. Ensure .persona is in .gitignore
+    _ensure_persona_in_gitignore()
+
+    # Check for session ID in environment variable if not specified in args
+    if not session_id and "SILICA_DEVELOPER_SESSION_ID" in os.environ:
+        session_id = os.environ.get("SILICA_DEVELOPER_SESSION_ID")
+
+    # Initialize user interface
     user_interface = CLIUserInterface(console, parsed_sandbox_mode)
 
     # Handle prompt loading from file or direct input (identical to original)
@@ -703,20 +1035,21 @@ def cyclopts_main(
         user_interface=user_interface,
         session_id=session_id,
         cli_args=original_args,
+        persona_base_directory=persona_obj.base_directory,
     )
 
-    # Setup system prompt/persona (identical to original)
-    system_block: dict[str, Any] | None = (
-        wrap_text_as_content_block(personas.for_name(persona)) if persona else None
-    )
+    # Set the agent context reference in the UI for keyboard shortcuts
+    user_interface.agent_context = context
 
-    # Run the agent loop (identical to original)
+    # Run the agent loop
+    # Pass the persona system_block - it will be used as fallback if persona.md doesn't exist
     asyncio.run(
         run(
             agent_context=context,
             initial_prompt=initial_prompt,
-            system_prompt=system_block,
+            system_prompt=persona_obj.system_block,
             single_response=bool(initial_prompt),
             enable_compaction=not disable_compaction,
+            log_file_path=log_requests,
         )
     )
