@@ -23,7 +23,12 @@ except ImportError:
 
 
 class Toolbox:
-    def __init__(self, context: AgentContext, tool_names: List[str] | None = None):
+    def __init__(
+        self,
+        context: AgentContext,
+        tool_names: List[str] | None = None,
+        tools: List[str] | None = None,
+    ):
         self.context = context
         self.local = {}  # CLI tools
 
@@ -34,6 +39,21 @@ class Toolbox:
         else:
             self.agent_tools = ALL_TOOLS
 
+        # Filter out persona tools if only built-in persona is active (no persona.md)
+        from pathlib import Path
+
+        if context.history_base_dir is not None and isinstance(
+            context.history_base_dir, Path
+        ):
+            persona_file = context.history_base_dir / "persona.md"
+            if not persona_file.exists():
+                # Built-in persona only - remove persona editing tools
+                self.agent_tools = [
+                    tool
+                    for tool in self.agent_tools
+                    if tool.__name__ not in ["read_persona", "write_persona"]
+                ]
+
         # Register CLI tools
         self.register_cli_tool("help", self._help, "Show help", aliases=["h"])
         self.register_cli_tool("tips", self._tips, "Show usage tips and tricks")
@@ -78,6 +98,12 @@ class Toolbox:
             aliases=["debug"],
         )
 
+        self.register_cli_tool(
+            "info",
+            self._info,
+            "Show statistics about the current session",
+        )
+
         self.register_cli_tool(
             "view-memory", self._launch_memory_webapp, "Launch memory webapp"
         )
@@ -93,6 +119,18 @@ class Toolbox:
             "resume", self._resume_session, "Resume a previous developer session"
         )
 
+        # Register compaction CLI tools
+        self.register_cli_tool(
+            "compact",
+            self._compact,
+            "Explicitly trigger full conversation compaction",
+        )
+        self.register_cli_tool(
+            "mc",
+            self._micro_compact,
+            "Micro-compact: summarize first N turns and keep the rest (default N=3)",
+        )
+
         # Register Google Auth CLI tools
         for name, tool_info in GOOGLE_AUTH_CLI_TOOLS.items():
             self.register_cli_tool(
@@ -138,7 +176,11 @@ class Toolbox:
             chat_history=chat_history or [],
         )
 
-        self.context.user_interface.handle_system_message(content, markdown=False)
+        # Render info command output as markdown for better formatting
+        render_as_markdown = name == "info"
+        self.context.user_interface.handle_system_message(
+            content, markdown=render_as_markdown
+        )
         add_to_buffer = confirm_to_add
         if confirm_to_add and content and content.strip():
             add_to_buffer = (
@@ -222,16 +264,11 @@ class Toolbox:
                 ]
 
                 # Execute in parallel with proper cancellation handling
-                try:
-                    parallel_results = await asyncio.gather(
-                        *parallel_coroutines, return_exceptions=True
-                    )
-                except (KeyboardInterrupt, asyncio.CancelledError):
-                    # Cancel all running tasks
-                    for coro in parallel_coroutines:
-                        if hasattr(coro, "cancel"):
-                            coro.cancel()
-                    raise KeyboardInterrupt("Tool execution interrupted by user")
+                # Note: asyncio.gather with return_exceptions=True will not raise exceptions
+                # but will instead return them in the results list
+                parallel_results = await asyncio.gather(
+                    *parallel_coroutines, return_exceptions=True
+                )
 
                 # Handle results and exceptions
                 for tool_use, result in zip(parallel_tools, parallel_results):
@@ -324,6 +361,10 @@ class Toolbox:
         help_text = "## Available commands:\n"
         help_text += "- **/restart** - Clear chat history and start over\n"
         help_text += "- **/quit** - Quit the chat\n"
+        help_text += (
+            "- **/compact** - Explicitly trigger full conversation compaction\n"
+        )
+        help_text += "- **/mc [N]** - Micro-compact: summarize first N turns (default 3) and keep the rest\n"
 
         displayed_tools = set()
         for tool_name, spec in self.local.items():
@@ -361,6 +402,12 @@ class Toolbox:
 * The AI can read, write, and edit files in your project
 * Use `/add` and `/remove` to manage which files are in the sandbox context
 
+**Thinking Mode (Extended Thinking API):**
+* Press **Ctrl+T** to cycle through thinking modes: off → 💭 normal (8k) → 🧠 ultra (20k) → off
+* When enabled, the AI thinks deeply before responding (costs 3x input pricing)
+* The prompt shows the current mode: `💭 $0.00 >` (normal) or `🧠 $0.00 >` (ultra)
+* Thinking content is displayed in a collapsible panel after responses
+
 **Command Shortcuts:**
 * Use `/exec` to run shell commands quickly
 * Use `/commit` to auto-generate git commit messages
@@ -372,6 +419,12 @@ class Toolbox:
 * Use `/resume <session-id>` to continue where you left off
 * Session history is automatically saved and organized by directory
 
+**Conversation Compaction:**
+* Use `/compact` to manually compress the entire conversation
+* Use `/mc [N]` to micro-compact just the first N turns (default 3) while keeping the rest
+* Compaction helps manage token usage in long conversations
+* Automatic compaction triggers at 65% of context window
+
 **Efficiency Tips:**
 * The AI can work with multiple files simultaneously
 * Ask for explanations of code, suggestions for improvements, or help debugging
@@ -426,14 +479,47 @@ class Toolbox:
             "\n\n[bold cyan]Chat History (with inlined file contents):[/bold cyan]\n"
         )
         inlined_history = _inline_latest_file_mentions(kwargs["chat_history"])
-        for message in inlined_history:
+        for msg_idx, message in enumerate(inlined_history):
+            content += f"\n\n[bold]Message {msg_idx} ({message['role']}):[/bold]"
+
             if isinstance(message["content"], str):
-                content += f"\n[bold]{message['role']}:[/bold] {message['content']}"
+                content += f"\n  [text] {message['content'][:100]}..."
             elif isinstance(message["content"], list):
-                content += f"\n[bold]{message['role']}:[/bold]"
-                for block in message["content"]:
-                    if isinstance(block, dict) and "text" in block:
-                        content += f"\n{block['text']}"
+                content += f"\n  Content blocks: {len(message['content'])}"
+                for block_idx, block in enumerate(message["content"]):
+                    # Get block type
+                    block_type = None
+                    if isinstance(block, dict):
+                        block_type = block.get("type", "unknown")
+                    elif hasattr(block, "type"):
+                        block_type = block.type
+                    else:
+                        block_type = type(block).__name__
+
+                    content += f"\n    [{block_idx}] {block_type}"
+
+                    # Show preview of content
+                    if isinstance(block, dict):
+                        if "text" in block:
+                            preview = block["text"][:100]
+                            content += f": {preview}{'...' if len(block['text']) > 100 else ''}"
+                        elif "thinking" in block:
+                            content += (
+                                f" (signature: {block.get('signature', 'N/A')[:20]}...)"
+                            )
+                        elif "tool_use" in block or block_type == "tool_use":
+                            content += f" (name: {block.get('name', 'N/A')})"
+                        elif "tool_result" in block or block_type == "tool_result":
+                            content += f" (tool_use_id: {block.get('tool_use_id', 'N/A')[:20]}...)"
+                    elif hasattr(block, "text"):
+                        preview = block.text[:100]
+                        content += (
+                            f": {preview}{'...' if len(block.text) > 100 else ''}"
+                        )
+                    elif hasattr(block, "thinking"):
+                        content += f" (signature: {block.signature[:20] if hasattr(block, 'signature') else 'N/A'}...)"
+                    elif hasattr(block, "name"):
+                        content += f" (name: {block.name})"
 
         return content
 
@@ -760,8 +846,11 @@ class Toolbox:
         # Extract optional workdir filter
         workdir = user_input.strip() if user_input.strip() else None
 
-        # Get the list of sessions'1583628'
-        sessions = list_sessions(workdir)
+        # Get history_base_dir from context (persona-aware)
+        history_base_dir = getattr(self.context, "history_base_dir", None)
+
+        # Get the list of sessions
+        sessions = list_sessions(workdir, history_base_dir=history_base_dir)
 
         # Print the formatted list
         print_session_list(sessions)
@@ -780,8 +869,11 @@ class Toolbox:
             )
             return "Error: No session ID provided"
 
+        # Get history_base_dir from context (persona-aware)
+        history_base_dir = getattr(self.context, "history_base_dir", None)
+
         # Attempt to resume the session
-        success = resume_session(session_id)
+        success = resume_session(session_id, history_base_dir=history_base_dir)
 
         if not success:
             return f"Failed to resume session {session_id}"
@@ -798,6 +890,165 @@ class Toolbox:
         # Return the result for display
         return result
 
+    def _info(self, user_interface, sandbox, user_input, *args, **kwargs):
+        """Show statistics about the current session."""
+        from datetime import datetime
+        from pathlib import Path
+
+        # Get session information
+        session_id = self.context.session_id
+        parent_session_id = self.context.parent_session_id
+
+        # Get persona name from history_base_dir
+        persona_name = "default"
+        if self.context.history_base_dir:
+            history_base_dir = (
+                Path(self.context.history_base_dir)
+                if not isinstance(self.context.history_base_dir, Path)
+                else self.context.history_base_dir
+            )
+            # Extract persona name from path like ~/.silica/personas/{persona_name}
+            if history_base_dir.parent.name == "personas":
+                persona_name = history_base_dir.name
+        else:
+            history_base_dir = Path.home() / ".silica" / "personas" / "default"
+
+        # Get model information
+        model_spec = self.context.model_spec
+        model_name = model_spec["title"]
+        max_tokens = model_spec["max_tokens"]
+        context_window = model_spec["context_window"]
+
+        # Get thinking mode
+        thinking_mode = self.context.thinking_mode
+        thinking_display = {
+            "off": "Off",
+            "normal": "💭 Normal (8k tokens)",
+            "ultra": "🧠 Ultra (20k tokens)",
+        }.get(thinking_mode, thinking_mode)
+
+        # Get usage summary
+        usage = self.context.usage_summary()
+        total_input_tokens = usage["total_input_tokens"]
+        total_output_tokens = usage["total_output_tokens"]
+        total_thinking_tokens = usage.get("total_thinking_tokens", 0)
+        cached_tokens = usage["cached_tokens"]
+        total_cost = usage["total_cost"]
+        thinking_cost = usage.get("thinking_cost", 0.0)
+
+        # Get message count
+        message_count = len(self.context.chat_history)
+
+        # Calculate conversation size if available
+        conversation_size = getattr(self.context, "_last_conversation_size", None)
+
+        # Get session creation and update times if available
+        history_dir = history_base_dir / "history"
+        context_dir = parent_session_id if parent_session_id else session_id
+        history_file = (
+            history_dir
+            / context_dir
+            / ("root.json" if not parent_session_id else f"{session_id}.json")
+        )
+
+        created_at = None
+        last_updated = None
+        root_dir = None
+
+        if history_file.exists():
+            try:
+                import json
+
+                with open(history_file, "r") as f:
+                    session_data = json.load(f)
+                    metadata = session_data.get("metadata", {})
+                    created_at = metadata.get("created_at")
+                    last_updated = metadata.get("last_updated")
+                    root_dir = metadata.get("root_dir")
+            except Exception:
+                pass
+
+        # Format the output
+        info = "# Session Information\n\n"
+
+        # Persona
+        info += f"**Persona:** `{persona_name}`\n\n"
+
+        # Session IDs
+        info += f"**Session ID:** `{session_id}`\n\n"
+        if parent_session_id:
+            info += f"**Parent Session ID:** `{parent_session_id}`\n\n"
+
+        # Session timestamps
+        if created_at:
+            try:
+                dt = datetime.fromisoformat(created_at.replace("Z", "+00:00"))
+                info += f"**Created:** {dt.strftime('%Y-%m-%d %H:%M:%S %Z')}\n\n"
+            except Exception:
+                info += f"**Created:** {created_at}\n\n"
+
+        if last_updated:
+            try:
+                dt = datetime.fromisoformat(last_updated.replace("Z", "+00:00"))
+                info += f"**Last Updated:** {dt.strftime('%Y-%m-%d %H:%M:%S %Z')}\n\n"
+            except Exception:
+                info += f"**Last Updated:** {last_updated}\n\n"
+
+        # Root directory
+        if root_dir:
+            info += f"**Working Directory:** `{root_dir}`\n\n"
+
+        # Model information
+        info += "## Model Configuration\n\n"
+        info += f"**Model:** {model_name}\n\n"
+        info += f"**Max Tokens:** {max_tokens:,}\n\n"
+        info += f"**Context Window:** {context_window:,} tokens\n\n"
+        info += f"**Thinking Mode:** {thinking_display}\n\n"
+
+        # Conversation statistics
+        info += "## Conversation Statistics\n\n"
+        info += f"**Message Count:** {message_count}\n\n"
+
+        if conversation_size:
+            usage_percentage = (conversation_size / context_window) * 100
+            info += f"**Conversation Size:** {conversation_size:,} tokens ({usage_percentage:.1f}% of context)\n\n"
+
+            # Calculate tokens remaining before compaction threshold (85%)
+            compaction_threshold = int(context_window * 0.85)
+            tokens_remaining = max(0, compaction_threshold - conversation_size)
+            info += f"**Tokens Until Compaction:** {tokens_remaining:,} (threshold: 85%)\n\n"
+
+        # Token usage
+        info += "## Token Usage\n\n"
+        info += f"**Input Tokens:** {total_input_tokens:,}"
+        if cached_tokens > 0:
+            info += f" (cached: {cached_tokens:,})"
+        info += "\n\n"
+        info += f"**Output Tokens:** {total_output_tokens:,}\n\n"
+
+        if total_thinking_tokens > 0:
+            info += f"**Thinking Tokens:** {total_thinking_tokens:,}\n\n"
+
+        total_tokens = total_input_tokens + total_output_tokens + total_thinking_tokens
+        info += f"**Total Tokens:** {total_tokens:,}\n\n"
+
+        # Cost information
+        info += "## Cost Information\n\n"
+        info += f"**Session Cost:** ${total_cost:.4f}\n\n"
+
+        if thinking_cost > 0:
+            info += f"**Thinking Cost:** ${thinking_cost:.4f}\n\n"
+            non_thinking_cost = total_cost - thinking_cost
+            info += f"**Non-Thinking Cost:** ${non_thinking_cost:.4f}\n\n"
+
+        # Cost breakdown by model if multiple models used
+        if len(usage["model_breakdown"]) > 1:
+            info += "### Cost Breakdown by Model\n\n"
+            for model, model_usage in usage["model_breakdown"].items():
+                info += f"- **{model}:** ${model_usage['total_cost']:.4f}\n\n"
+
+        return info
+
     def _model(self, user_interface, sandbox, user_input, *args, **kwargs):
         """Display or change the current AI model"""
         from .models import model_names, get_model, MODEL_MAP
@@ -873,6 +1124,164 @@ class Toolbox:
 
             return error_msg
 
+    def _compact(self, user_interface, sandbox, user_input, *args, **kwargs):
+        """Explicitly trigger full conversation compaction."""
+        from silica.developer.compacter import ConversationCompacter
+        import anthropic
+        import os
+        from dotenv import load_dotenv
+
+        # Check if there's enough conversation to compact
+        if len(self.context.chat_history) <= 2:
+            return "Error: Not enough conversation history to compact (need more than 2 messages)"
+
+        # Create Anthropic client and compacter instance
+        load_dotenv()
+        api_key = os.getenv("ANTHROPIC_API_KEY")
+        if not api_key:
+            return "Error: ANTHROPIC_API_KEY environment variable not set"
+
+        client = anthropic.Client(api_key=api_key)
+        compacter = ConversationCompacter(client=client)
+        model_name = self.context.model_spec["title"]
+
+        try:
+            # Force compaction
+            user_interface.handle_system_message(
+                "Compacting conversation (this may take a moment)...", markdown=False
+            )
+
+            metadata = compacter.compact_conversation(
+                self.context, model_name, force=True
+            )
+
+            if metadata:
+                # Build result message
+                result = "✓ Conversation compacted successfully!\n\n"
+                result += f"**Original:** {metadata.original_message_count} messages ({metadata.original_token_count:,} tokens)\n\n"
+                result += f"**Compacted:** {metadata.compacted_message_count} messages ({metadata.summary_token_count:,} tokens)\n\n"
+                result += f"**Compression ratio:** {metadata.compaction_ratio:.1%}\n\n"
+                result += f"**Archive:** {metadata.archive_name}\n\n"
+
+                # Flush the compacted context
+                self.context.flush(self.context.chat_history, compact=False)
+
+                return result
+            else:
+                return "Error: Compaction failed to generate metadata"
+
+        except Exception as e:
+            import traceback
+
+            error_details = traceback.format_exc()
+            user_interface.handle_system_message(
+                f"Compaction failed: {e}\n\n{error_details}", markdown=False
+            )
+            return f"Error: Compaction failed - {e}"
+
+    def _micro_compact(self, user_interface, sandbox, user_input, *args, **kwargs):
+        """Micro-compact: summarize first N turns and keep the rest."""
+        from silica.developer.compacter import ConversationCompacter
+        from silica.developer.context import AgentContext
+
+        # Parse the number of turns from user_input
+        turns_to_compact = 3  # default
+        if user_input.strip():
+            try:
+                turns_to_compact = int(user_input.strip())
+                if turns_to_compact < 1:
+                    return "Error: Number of turns must be at least 1"
+            except ValueError:
+                return f"Error: Invalid number '{user_input.strip()}'. Please provide an integer."
+
+        # Calculate number of messages for N turns
+        # Turn structure: must start with user and end with user
+        # Turn 1: 1 message (user)
+        # Turn 2: 3 messages (user, assistant, user)
+        # Turn 3: 5 messages (user, assistant, user, assistant, user)
+        # Turn N: (2N - 1) messages
+        messages_to_compact = (turns_to_compact * 2) - 1
+
+        # Check if there's enough conversation to compact
+        if len(self.context.chat_history) <= messages_to_compact:
+            return f"Error: Not enough conversation history to micro-compact {turns_to_compact} turns (need more than {messages_to_compact} messages, have {len(self.context.chat_history)})"
+
+        # Separate messages to compact from messages to keep
+        messages_to_summarize = self.context.chat_history[:messages_to_compact]
+        messages_to_keep = self.context.chat_history[messages_to_compact:]
+
+        # Create Anthropic client and compacter instance
+        import anthropic
+        import os
+        from dotenv import load_dotenv
+
+        load_dotenv()
+        api_key = os.getenv("ANTHROPIC_API_KEY")
+        if not api_key:
+            return "Error: ANTHROPIC_API_KEY environment variable not set"
+
+        client = anthropic.Client(api_key=api_key)
+        compacter = ConversationCompacter(client=client)
+        model_name = self.context.model_spec["title"]
+
+        try:
+            user_interface.handle_system_message(
+                f"Micro-compacting first {turns_to_compact} turns (this may take a moment)...",
+                markdown=False,
+            )
+
+            # Create a temporary context with just the messages to summarize
+            # This allows us to reuse the existing generate_summary method
+            temp_context = AgentContext(
+                parent_session_id=self.context.parent_session_id,
+                session_id=self.context.session_id,
+                model_spec=self.context.model_spec,
+                sandbox=self.context.sandbox,
+                user_interface=self.context.user_interface,
+                usage=self.context.usage,
+                memory_manager=self.context.memory_manager,
+                history_base_dir=self.context.history_base_dir,
+            )
+            temp_context._chat_history = messages_to_summarize
+
+            # Use the existing generate_summary method
+            summary_obj = compacter.generate_summary(temp_context, model_name)
+            summary = summary_obj.summary
+
+            # Create new message history with summary + kept messages
+            new_messages = [
+                {
+                    "role": "user",
+                    "content": f"### Micro-Compacted Summary (first {turns_to_compact} turns)\n\n{summary}\n\n---\n\nContinuing with remaining conversation...",
+                }
+            ]
+            new_messages.extend(messages_to_keep)
+
+            # Update the context in place
+            self.context._chat_history = new_messages
+            self.context._tool_result_buffer.clear()
+
+            # Flush the updated context
+            self.context.flush(self.context.chat_history, compact=False)
+
+            # Build result message
+            result = "✓ Micro-compaction completed!\n\n"
+            result += f"**Compacted:** First {turns_to_compact} turns ({messages_to_compact} messages)\n\n"
+            result += f"**Kept:** {len(messages_to_keep)} messages from the rest of the conversation\n\n"
+            result += f"**Final message count:** {len(new_messages)} (was {len(self.context.chat_history) + messages_to_compact})\n\n"
+            result += f"**Estimated compression:** {messages_to_compact} messages → ~{summary_obj.summary_token_count:,} tokens\n\n"
+
+            return result
+
+        except Exception as e:
+            import traceback
+
+            error_details = traceback.format_exc()
+            user_interface.handle_system_message(
+                f"Micro-compaction failed: {e}\n\n{error_details}", markdown=False
+            )
+            return f"Error: Micro-compaction failed - {e}"
+
     def schemas(self, enable_caching: bool = True) -> List[dict]:
         """Generate schemas for all tools in the toolbox.
 
