@@ -12,6 +12,17 @@ from .shell import (
     shell_session_set_timeout,
 )
 from .web import web_search, safe_curl
+from .browser_session_tools import (
+    get_browser_capabilities,
+    browser_session_create,
+    browser_session_navigate,
+    browser_session_interact,
+    browser_session_inspect,
+    browser_session_screenshot,
+    browser_session_list,
+    browser_session_destroy,
+    browser_session_get_info,
+)
 from .gcal import (
     calendar_setup,
     calendar_list_events,
@@ -48,6 +59,14 @@ from .github import GITHUB_TOOLS
 from .github_comments import (
     GITHUB_COMMENT_TOOLS,
 )
+from .persona_tools import (
+    read_persona,
+    write_persona,
+)
+from .webcam import (
+    get_webcam_capabilities,
+    webcam_snapshot,
+)
 
 ALL_TOOLS = (
     [
@@ -60,6 +79,20 @@ ALL_TOOLS = (
         agent,
         safe_curl,
         python_repl,
+        # Browser tools
+        get_browser_capabilities,
+        # Browser session tools
+        browser_session_create,
+        browser_session_navigate,
+        browser_session_interact,
+        browser_session_inspect,
+        browser_session_screenshot,
+        browser_session_list,
+        browser_session_destroy,
+        browser_session_get_info,
+        # Webcam tools
+        get_webcam_capabilities,
+        webcam_snapshot,
         # Shell tools (dual architecture)
         shell_execute,
         shell_session_create,
@@ -90,6 +123,8 @@ ALL_TOOLS = (
         todo_write,
         list_sessions_tool,
         get_session_tool,
+        read_persona,
+        write_persona,
     ]
     + GITHUB_TOOLS
     + GITHUB_COMMENT_TOOLS
