@@ -1,4 +1,5 @@
 import json
+import shutil
 from pathlib import Path
 from typing import Optional, Dict, Any
 
@@ -8,6 +9,32 @@ from silica.developer.tools.framework import tool
 from silica.developer.utils import render_tree
 
 
+# Cache ripgrep availability check for efficiency (checked once per process startup)
+_RIPGREP_AVAILABLE = None
+
+
+def _has_ripgrep() -> bool:
+    """Check if ripgrep (rg) is available on the system.
+
+    This function caches the result to avoid repeated expensive shutil.which() calls
+    since ripgrep availability is unlikely to change during process lifetime.
+    """
+    global _RIPGREP_AVAILABLE
+    if _RIPGREP_AVAILABLE is None:
+        _RIPGREP_AVAILABLE = shutil.which("rg") is not None
+    return _RIPGREP_AVAILABLE
+
+
+def _refresh_ripgrep_cache() -> None:
+    """Refresh the cached ripgrep availability check.
+
+    This can be called if ripgrep gets installed/uninstalled during runtime,
+    though this is rare in practice.
+    """
+    global _RIPGREP_AVAILABLE
+    _RIPGREP_AVAILABLE = None
+
+
 @tool
 def get_memory_tree(
     context: "AgentContext", prefix: Optional[str] = None, depth: int = -1
@@ -44,14 +71,27 @@ def get_memory_tree(
 async def search_memory(
     context: "AgentContext", query: str, prefix: Optional[str] = None
 ) -> str:
-    """Search memory with the given query.
+    """Search memory entries for content matching the given query using intelligent text search.
+
+    This tool performs semantic and text-based search across all memory entries using ripgrep
+    (if available) or grep as fallback, returning relevant matches with context about why they match.
 
     Args:
-        query: Search query
-        prefix: Optional path prefix to limit search scope
+        query: Search terms or phrases to find in memory content
+        prefix: Optional path to limit search to a specific memory subtree
 
     Returns:
-        a list of memory paths (these should be used for read/write_memory_entry tools.
+        Formatted search results showing:
+        - Memory paths that contain matching content
+        - Brief explanations of why each entry matches
+        - Context snippets around matches
+        - "No matching memory entries found" if no results
+
+    Search Tips:
+        - Use specific keywords from the content you're looking for
+        - Try different variations (e.g., "React" vs "react component")
+        - Search is case-insensitive and supports partial matches
+        - Uses ripgrep (rg) when available for faster, more accurate results
     """
     memory_dir = context.memory_manager.base_dir
     search_path = memory_dir
@@ -62,42 +102,81 @@ async def search_memory(
             return f"Error: Path {prefix} does not exist or is not a directory"
 
     try:
-        # Use the agent tool to kick off an agentic search using grep
+        # Use the agent tool to kick off an agentic search using ripgrep or grep
         from silica.developer.tools.subagent import agent
 
+        has_rg = _has_ripgrep()
+
+        if has_rg:
+            search_tool_intro = "You have ripgrep (rg) available."
+            search_commands = """
+        Use these ripgrep command patterns:
+        - `rg "SEARCH_TERM" /path/to/search --type md` (search in .md files)
+        - `rg -i "SEARCH_TERM" /path/to/search --type md` (case insensitive search)
+        - `rg -l "SEARCH_TERM" /path/to/search --type md` (just list matching files)
+        - `rg -n "SEARCH_TERM" /path/to/search --type md` (show line numbers)
+        - `rg -C 2 "SEARCH_TERM" /path/to/search --type md` (show 2 lines of context around matches)
+        - `rg "TERM1|TERM2" /path/to/search --type md` (search for multiple terms with OR)
+        - `rg "TERM1.*TERM2" /path/to/search --type md` (search for terms in sequence)
+        """
+        else:
+            search_tool_intro = "Ripgrep is not available, using grep."
+            search_commands = """
+        Use these grep command patterns:
+        - `grep -r --include="*.md" "SEARCH_TERM" /path/to/search`
+        - `grep -r --include="*.md" -i "SEARCH_TERM" /path/to/search` (case insensitive)
+        - `grep -r --include="*.md" -l "SEARCH_TERM" /path/to/search` (just list files)
+        - `grep -r --include="*.md" -n "SEARCH_TERM" /path/to/search` (show line numbers)
+        - `grep -r --include="*.md" -A 2 -B 2 "SEARCH_TERM" /path/to/search` (show 2 lines of context)
+        - `grep -r --include="*.md" -E "TERM1|TERM2" /path/to/search` (search for multiple terms with OR)
+        - `grep -r --include="*.md" "TERM1.*TERM2" /path/to/search` (search for terms in sequence)
+        """
+
         prompt = f"""
-        You are an expert in using grep to search through files. 
+        You are an expert in file searching and memory system navigation.
         
-        TASK: Search through memory entries in the directory "{search_path}" to find information relevant to this query: "{query}"
+        SEARCH TOOL STATUS: {search_tool_intro}
         
         The memory system stores entries as:
-        1. .md files for content
-        2. .metadata.json files for metadata
+        1. .md files for content (search these)
+        2. .metadata.json files for metadata (usually skip these unless specifically needed)
         
-        Use grep to search through the .md files and find matches for the query.
+        {search_commands}
         
-        Here are some grep commands you might use:
-        - `grep -r --include="*.md" "{query}" {search_path}`
-        - `grep -r --include="*.md" -i "{query}" {search_path}` (case insensitive)
-        - `grep -r --include="*.md" -l "{query}" {search_path}` (just list files)
+        IMPORTANT SEARCH FORMATTING RULES:
+        - Format queries for grep/ripgrep pattern matching (not generic text search)
+        - Use exact word matching when appropriate: "\\\\bword\\\\b" 
+        - Use case-insensitive search (-i) for better recall
+        - For multiple terms, use OR patterns: "term1|term2"
+        - For phrase matching, use exact quotes: "exact phrase"
+        - For wildcard matching, use appropriate regex: "term.*pattern"
         
-        After finding matches, examine the matching files to provide context around the matches. Format your results as:
+        SEARCH STRATEGY (MAXIMUM 2 ATTEMPTS):
+        1. First attempt: Use the most specific, well-formatted search terms
+        2. If no results, make ONE more attempt with broader/alternative terms
+        3. After 2 attempts with no results, stop and report failure
+        
+        YOUR TASK: Search for information relevant to this query and path:
+        - Search Query: {query}
+        - Search Path: {search_path}
+        
+        After finding matches, examine the matching files to provide context. Format your results as:
         
         ## Search Results
         
-        1. [Path to memory]: Brief explanation of why this matches
-        2. [Path to memory]: Brief explanation of why this matches
+        1. **[memory/path]**: Brief explanation of why this matches and what it contains
+        2. **[memory/path]**: Brief explanation of why this matches and what it contains
         
-        For the paths, strip off the .md extension and the base directory path to present clean memory paths.
+        For the paths, strip off the .md extension and the base directory path to present clean memory paths that can be used with read_memory_entry.
         
-        If no results match, say "No matching memory entries found."
+        If no results match after 2 attempts, say "No matching memory entries found." and suggest trying different search terms.
         """
 
         # Use the subagent tool to perform the search with shell_execute tool
         result = await agent(
             context=context,
             prompt=prompt,
-            tool_names="shell_execute",  # Allow grep commands
+            tool_names="shell_execute",  # Allow ripgrep/grep commands
             model="smart",
         )
 
